#ifndef CANSubDataWrapper_WRAPPER_HEADER_H
#define CANSubDataWrapper_WRAPPER_HEADER_H


#include "sub/SubDataWrapper.h"
#include "can_msg_node/executable.h"
#include "../Messages.h"

enum PendingSignals
{
	IPC_Target_Motor_En,
    IPC_Target_Gear,
    IPC_Target_Mode,
    IPC_Life_Signal,
    IPC_Target_Speed,
    IPC_Target_SpdValid,
    IPC_Target_Brk_Press,
    IPC_Target_Angle,
    IPC_Target_Steering_En,
    IPC_Target_Torque,
    IPC_Torque_Rpm_Mode,
    IPC_TurnLight,
    IPC_Target_SpdMode
};

inline void control_request1_msg_parser(SWC2CAN message, physical_control_request1_t* _control_request1_msg)
{
	switch(message.signal)
	{
	        case IPC_Target_Motor_En:
	                _control_request1_msg->ipc_target_motor_en = (uint8_t)message.value;
	                break;
	        case IPC_Target_Gear:
	                _control_request1_msg->ipc_target_gear = (uint8_t)message.value;
	                break;
	        case IPC_Target_Mode:
	                _control_request1_msg->ipc_target_mode = (uint8_t)message.value;
	                break;
	        case IPC_Life_Signal:
	                _control_request1_msg->ipc_life_signal = (uint8_t)message.value;
	                break;
	        case IPC_Target_Speed:
					double temp_target_speed;
					memcpy(&temp_target_speed, &message.value, sizeof(double));
	                _control_request1_msg->ipc_target_speed  = (float)temp_target_speed;
	                break;
	        case IPC_Target_SpdValid:
	                _control_request1_msg->ipc_target_spd_valid = (uint8_t)message.value;
	                break;
	        case IPC_Target_Brk_Press:
					double temp_brk_press;
					memcpy(&temp_brk_press, &message.value, sizeof(double));
	                _control_request1_msg->ipc_target_brk_press  = (float)temp_brk_press;
	                break;
	        case IPC_Target_Angle:
	        		double temp;
	        		memcpy(&temp, &message.value, 8);
	                _control_request1_msg->ipc_target_angle = (float)temp;
//	        		memcpy(&_control_request1_msg->ipc_target_angle, &message.value, 4);
//	                printf("\n\n in sc = %f = %ud\n\n\n", _control_request1_msg->ipc_target_angle, message.value);
	                break;
	        case IPC_Target_Steering_En:
	                _control_request1_msg->ipc_target_steering_en = (uint8_t)message.value;
	                break;
	        case IPC_Target_Torque:
	        		double temp1;
	        		memcpy(&temp1, &message.value, 8);
	                _control_request1_msg->ipc_target_torque = (float)temp1;
	                break;
	        case IPC_Torque_Rpm_Mode:
	                _control_request1_msg->ipc_torque_rpm_mode = (uint8_t)message.value;
	                break;
	        case IPC_TurnLight:
	                _control_request1_msg->ipc_turn_light = (uint8_t)message.value;
	                break;
	        case IPC_Target_SpdMode:
	                _control_request1_msg->ipc_target_spd_mode = (uint8_t)message.value;
	                break;
	        default:
	                break;

	}
}



template <typename DataType>
class Lat2CAN_SubDataWrapper : public SubDataWrapper<DataType>
{
	physical_control_request1_t* _control_request1;
	SWC2CAN _lat_data;
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
	{
		_lat_data = (SWC2CAN)*(ptr->Get());
		control_request1_msg_parser(_lat_data,_control_request1);
		// std::cout << "Received signal data from Lat" << std::endl;

		//std::cout << "+++++CAN sig: " <<+_lat_data.signal << ", value: " <<_lat_data.value<<"Control Req : "<< _control_request1->ipc_target_angle<< "\n";
		if (_lat_data.signal == 9) {

			std::cout << "+++++OP torque in float: " << _control_request1->ipc_target_torque << "\n";
		}

	}
public:
	const SWC2CAN& getLatData()
	{
		return _lat_data;
	}

	void set_control_req_msg(physical_control_request1_t* input_msg)
	{
		_control_request1 = input_msg;
	}
};

template <typename DataType>
class Long2CAN_SubDataWrapper : public SubDataWrapper<DataType>
{
	physical_control_request1_t* _control_request1;
	SWC2CAN _long_data;
	void Process(ara::com::SamplePtr<DataType const>* ptr) override
    {
		_long_data = (SWC2CAN)*(ptr->Get());
		control_request1_msg_parser(_long_data,_control_request1);
//        std::cout << "Received signal data from Long" << std::endl;
		if (_long_data.signal == 9) {

					std::cout << "+++++OP torque in float: " << _control_request1->ipc_target_torque << "\n";
			}
    }
public:
	const SWC2CAN& getLongData()
	{
		return _long_data;
	}

	void set_control_req_msg(physical_control_request1_t* input_msg)
	{
		_control_request1 = input_msg;
	}
};

#endif
